/*
 Sketch to run a stepper motor usig a load cell to adjust the speed.
 Arduino pin 2 -> HX711 CLK
 Arduino pin 3 -> HX711 DOUT
 Arduino pin 5V -> HX711 VCC
 Arduino pin GND -> HX711 GND
*/
#include "HX711.h"
//
//    Instantiate scale object
//
HX711 scale(2, 3);            // Load cell pins
float units;                  // Tension in grams
//
//    Declare motor variables
//
int reverseSwitch = 4;        // Hall swich for reverse
int driverPUL = 5;            // PUL- pin
int driverDIR = 6;            // DIR- pin
int pot = A0;                 // Potentiometer
int pd;                       // Pulse Delay period
boolean setdir = LOW;         // Set Direction
//
//    Limits
//
int scaleMin = 0;             // Lower scale limit is 0 grams
int scaleMax = 5000;          // Upper scale limit is 5000 grams
int pulseMin = 30;            // Notional fastest speed
int pulseMax = 32000;         // Notional minimum (30 seconds between pulses)
int potMin = 0;               // Potentiometer minimum value
int potMax = 1023;            // Potentiometer maximum speed
int potPD;                    // Max sepeed as set by potentiomenet
//
//    Motor reversal interrupt
//
void revmotor ()
{
  setdir = !setdir;
}
//
//    Initialization
//
void setup() {
  //
  //    Initialize load cell
  //
  Serial.begin(9600);
  Serial.println("HX711 calibration sketch");
  Serial.println("Remove all weight from scale");
  Serial.println("After readings begin, place known weight on scale");
  Serial.println("Press + or a to increase calibration factor");
  Serial.println("Press - or z to decrease calibration factor");
  scale.set_scale();
  scale.tare();  //Reset the scale to 0
  long zero_factor = scale.read_average(); //Get a baseline reading
  Serial.print("Zero factor: "); //This can be used to remove the need to tare the scale. Useful in permanent scale projects.
  Serial.println(zero_factor);
  //
  //    Initialize Motor
  //
  pinMode (driverPUL, OUTPUT);
  pinMode (driverDIR, OUTPUT);
  attachInterrupt(digitalPinToInterrupt(reverseSwitch), revmotor, FALLING);
}
//
//    Execution
//
void loop() {
  //
  //    Determine max speed set by potentiometer. Since there is no
  //      information supplied, I aam assuming that the pot will be
  //      used so that an increase in cuurrent implies faster speed
  //      for the stepper.
  //
  potPD = map(analogRead(pot),potMin,potMax,pulseMax,pulseMin);
  //
  //    Mow we need to determine the speed based on the tension.
  //    For this, we'll map the current tension reading into the
  //    speed range from the pot max calculated above.
  //    Lower tension implies higher speed
  //
  units = scale.get_units();          // Read scales
  units = (units>0) ? units : 0.0;    // Force into non-negative value
  pd = map(units,scaleMin,scaleMax,potPD,pulseMax);
  //
  //  Pulse the motor once using the calcuated delay
  //
  digitalWrite(driverPUL,HIGH);
  delayMicroseconds(pd);
  digitalWrite(driverPUL,LOW);
  delayMicroseconds(pd);
  digitalWrite(driverDIR,setdir);
  //
  //    Delay for 1/10 second for testing. This will avoid extremely
  //      fast or violent motor activity. This can later be adjusted
  //      (or deleted) as required.
  //
  delay(100);
 }